/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd

import com.kotlindiscord.kord.extensions.ExtensibleBot
import dev.kord.common.Color
import dev.kord.common.entity.PresenceStatus
import dev.kord.core.behavior.interaction.response.respond
import dev.kord.core.event.gateway.ReadyEvent
import dev.kord.core.event.interaction.ButtonInteractionCreateEvent
import dev.kord.core.on
import dev.kord.gateway.Intent
import dev.kord.gateway.PrivilegedIntent
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.Job
import kotlinx.coroutines.launch
import net.moonleay.botendo.build.BuildConstants
import net.moonleay.lilJudd.buttons.component.EditButtonManager
import net.moonleay.lilJudd.data.CredentialManager
import net.moonleay.lilJudd.data.DB
import net.moonleay.lilJudd.extensions.FeatureManageExtension
import net.moonleay.lilJudd.extensions.MatchExtension
import net.moonleay.lilJudd.extensions.SendPlannerExtension
import net.moonleay.lilJudd.extensions.VersionExtension
import net.moonleay.lilJudd.features.AvailabilityManager
import net.moonleay.lilJudd.features.MatchManager
import net.moonleay.lilJudd.features.TimeManager
import net.moonleay.lilJudd.util.Logger
import net.moonleay.lilJudd.util.MessageUtil
import kotlin.system.exitProcess

object Bot {
    //The kord object gets set at app launch
    lateinit var bot: ExtensibleBot

    private val jobs = mutableListOf<Job>()

    @OptIn(PrivilegedIntent::class)
    suspend fun start() {
        Logger.out("Starting Bot...")

        // Load config
        CredentialManager.load()

        // Don't run the bot when there is no bot token in config
        if (CredentialManager.token == "empty") {
            Logger.out("The config does not contain a bot token.")
            exitProcess(3)
        }

        // Check if the credentials for the Database are existent, don't run if they are missing
        if (CredentialManager.dbDomain == "empty" || CredentialManager.dbName == "empty" || CredentialManager.dbUser == "empty" || CredentialManager.dbPassword == "empty") {
            Logger.out("The config does not contain the whole Database credentials.")
            exitProcess(3)
        }

        // Connect to the database
        DB.connect(
            CredentialManager.dbDomain,
            CredentialManager.dbName,
            CredentialManager.dbUser,
            CredentialManager.dbPassword
        )

        // Register all the jobs
        jobs.addAll(
            listOf(
                TimeManager,
                AvailabilityManager,
            ).map {
                CoroutineScope(Dispatchers.Default).launch {
                    it.registerThread()
                }
            }
        )
        // Thanks silenium-dev <3

        // Create the bot object
        bot = ExtensibleBot(CredentialManager.token) {
            applicationCommands {
                enabled = true
            }

            extensions {
                add(::VersionExtension)
                add(::FeatureManageExtension)
                add(::SendPlannerExtension)
                add(::MatchExtension)
                //add(::UpdateRolesExtension) // This command is only for debugging purposes
                //add(::TestExtension) // See comment in TestExtension.kt
            }

            this.presence {
                this.status = PresenceStatus.DoNotDisturb
                this.playing("v." + BuildConstants.version)
            }

            this.intents {
                +Intent.GuildMembers
            }

            // Will add Sharding someday, I promise
            /*
            sharding { recommended ->
                Shards(recommended)
            } */
            // Same goes for a Database table rewrite
        }

        // Register button presses
        bot.kordRef.on<ButtonInteractionCreateEvent> {
            val inter = this.interaction
            val u = inter.user
            Logger.out("Button interaction: ${inter.componentId} from ${u.asUser().username}#${u.asUser().discriminator}")
            if (inter.componentId.startsWith("public.edit.")) {
                val response = inter.deferPublicMessageUpdate()
                val g = this.interaction.getOriginalInteractionResponse().getGuild()
                for (b in EditButtonManager.buttons) {
                    if (b.id != inter.componentId)
                        continue
                    b.onInteraction(inter, response, g, u)
                    return@on
                }
                return@on
            }
            if (inter.componentId.startsWith("public.message.")) {
                val response = inter.deferPublicResponse()
                val g = this.interaction.getOriginalInteractionResponse().getGuild()
                response.respond {
                    this.embeds = mutableListOf(
                        MessageUtil.getEmbed(
                            Color(0xE0311A),
                            "Error",
                            "Could not find button with id \"${inter.componentId}\".\nPlease report this.",
                            u.asUser().username + "#" + u.asUser().discriminator
                        )
                    )
                }
            }
        }

        bot.kordRef.on<ReadyEvent> {
            AvailabilityManager.runThread() // Update Availabilities
            MatchManager.update() // Update Matches
        }


        //Start the bot
        bot.start()
    }
}
